﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.GroupingVector {
	[Item("Grouping Vector Creator", "Creator class used to create GroupingVector solutions for bin packing problems.")]
	[StorableClass]
	public class GroupingVectorRandomCreator : PackingSolutionCreator, IStochasticOperator {

		public ILookupParameter<IRandom> RandomParameter {
			get { return (LookupParameter<IRandom>)Parameters["Random"]; }
		}
		public IValueLookupParameter<IntValue> PackingItemsParameter {
			get { return (IValueLookupParameter<IntValue>)Parameters["PackingItems"]; }
		}
		public IValueLookupParameter<IntValue> LowerBoundParameter {
			get { return (IValueLookupParameter<IntValue>)Parameters["LowerBound"]; }
		}

		[StorableConstructor]
		protected GroupingVectorRandomCreator(bool deserializing) : base(deserializing) { }
		protected GroupingVectorRandomCreator(GroupingVectorRandomCreator original, Cloner cloner) : base(original, cloner) {
		}
		public override IDeepCloneable Clone(Cloner cloner) {
			return new GroupingVectorRandomCreator(this, cloner);
		}

		public GroupingVectorRandomCreator()
			: base() {
			Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator."));
			Parameters.Add(new ValueLookupParameter<IntValue>("PackingItems", "The number of packing-items handled in this problem instance."));
			Parameters.Add(new ValueLookupParameter<IntValue>("LowerBound", "The lower possible number of bins needed to solve this problem (taken from Dell'Amico, Martello and Vigo; 2002)"));

			EncodedSolutionParameter.ActualName = "GroupingVector";
		}


		public static GroupingVectorEncoding Apply(int items, int lowerBound, IRandom random) {
			var solution = new GroupingVectorEncoding();
			solution.GroupingVector = new IntegerVector(items, random, 0, lowerBound);
			return solution;
		}

		protected override IPackingSolutionEncoding CreateSolution() {
			return Apply(PackingItemsParameter.ActualValue.Value, LowerBoundParameter.ActualValue.Value, RandomParameter.ActualValue);
		}
	}
}
